<?php

declare(strict_types=1);

namespace Dedi\SyliusPayboxPlugin\Payum\Action;

use Dedi\SyliusPayboxPlugin\Payum\PayboxApi;
use Payum\Core\Action\ActionInterface;
use Payum\Core\ApiAwareInterface;
use Payum\Core\ApiAwareTrait;
use Payum\Core\Exception\RequestNotSupportedException;
use Payum\Core\Reply\HttpResponse;
use Payum\Core\Request\Notify;
use Psr\Log\LoggerAwareInterface;
use Psr\Log\LoggerInterface;
use SM\Factory\FactoryInterface;
use SM\SMException;
use Sylius\Component\Core\Model\PaymentInterface;
use Sylius\Component\Payment\PaymentTransitions;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\HttpFoundation\Response;

class NotifyAction implements ActionInterface, ApiAwareInterface, LoggerAwareInterface
{
    use ApiAwareTrait;

    protected LoggerInterface $logger;

    public function __construct(private readonly RequestStack $requestStack, private readonly FactoryInterface $stateMachineFactory)
    {
        $this->apiClass = PayboxApi::class;
    }

    public function execute($request)
    {
        RequestNotSupportedException::assertSupports($this, $request);

        $status = $this->requestStack->getCurrentRequest()->get('error_code', 'null');
        if (null == $status) {
            throw new HttpResponse('', Response::HTTP_NO_CONTENT);
        }

        /** @var PaymentInterface $payment */
        $payment = $request->getFirstModel();
        $order = $payment->getOrder();

        // Get last payment if multi failed from Paybox form
        $lastPayment = $order->getLastPayment(PaymentInterface::STATE_NEW);
        if (null != $lastPayment && $lastPayment->getId() !== $payment->getId()) {
            $payment = $lastPayment;
        }

        $stateMachine = $this->stateMachineFactory->get($payment, PaymentTransitions::GRAPH);
        $query = $this->requestStack->getCurrentRequest()->getQueryString();

        try {
            if ($this->api->isAccepted($status)) {
                $stateMachine->apply(PaymentTransitions::TRANSITION_COMPLETE);
            } elseif ($this->api->isFailed($status)) {
                $stateMachine->apply(PaymentTransitions::TRANSITION_FAIL);
            } else {
                $stateMachine->apply(PaymentTransitions::TRANSITION_CANCEL);
            }
        } catch (SMException $e) {
            $this->logger->error(
                sprintf('[%s] %s An error occurred : %s', PayboxApi::CODE, self::class, $e->getMessage()),
                ['response' => $query, 'payment' => $payment->getDetails()],
            );
        }

        $this->logger->info(
            sprintf('[%s] %s', PayboxApi::CODE, self::class),
            ['response' => $query, 'payment' => $payment->getDetails()],
        );

        throw new HttpResponse('', Response::HTTP_OK);
    }

    public function setLogger(LoggerInterface $logger)
    {
        $this->logger = $logger;
    }

    public function supports($request)
    {
        return
            $request instanceof Notify &&
            $request->getFirstModel() instanceof PaymentInterface
        ;
    }
}
